-- einfache  Testbench fuer das Tastaturinterface und FIFO

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
use work.all;

entity testbench is
end entity testbench;


architecture behave of testbench is

	constant WIDTH : natural := 8; -- Breite des Tataturcodes
	constant fifo_depth : natural := 17; -- Kapazipaet des FIFO-Puffers
	signal clk : std_logic; -- Systemtakt
	signal tclk : std_logic; -- Tastaturtakt
	signal tdata : std_logic; -- Daten von der Tastatur
	signal rd_ena : std_logic; -- Daten wurden von read aus FIFO gelesen
	signal wr_ena : std_logic; -- Daten koennen in die FIFO uebernommen werden
	signal data_out : std_logic_vector(WIDTH-1 downto 0); -- Tastaturcode, parallel
	signal data_valid : std_logic; -- gueltiger Tastaturcode steht bereit
	signal parity_err : std_logic; -- Paritaets-Fehler, wenn ='1'
	signal clr_n : std_logic :='0'; -- Systemreset am Anfang
	signal data_in_fifo : std_logic_vector(WIDTH downto 0); -- parity_err + tastaturcode
	signal data_out_fifo : std_logic_vector(WIDTH downto 0); -- parity_err + tastaturcode
	signal empty : std_logic; -- FIFO ist leer
	signal full : std_logic; -- FIFO ist voll
	signal read_enable : std_logic; -- Daten wurden von read aus der FIFO gelesen

begin -- architecture behave

	Systemreset: process
	begin
		clr_n <= '1' after 10 us;
		wait;
	end process Systemreset;

	Systemtakt: entity taktgenerator 
		generic map(10 us,10 us)
		port map(
			clk_gen => clk
		);

	Tastatur: entity keybrd
		port map(
			tclk => tclk,
			tdata => tdata
		);

	-- DUT:
	Tastatur_interface: entity interface
		generic map(WIDTH)
		port map(
			clk => clk,
			clr_n => clr_n,
			tclk => tclk,
			tdata => tdata,
			rd_ena => wr_ena,
			data_out => data_out,
			data_valid => data_valid,
			parity_err => parity_err
		);
		
	wr_ena <= data_valid and not full;
	data_in_fifo(width) <= parity_err;
	data_in_fifo(width-1 downto 0) <= data_out;

	FIFO_Puffer: entity fifo(struct)
		generic map(width + 1, fifo_depth) -- +1 fr parity_err
		port map(
			clk => clk,
			resetn => clr_n,
			wr_ena => wr_ena,
			data_in => data_in_fifo,
			data_out => data_out_fifo,
			empty => empty,
			full => full,
			rd_ena => rd_ena
		);

	read_enable <= not empty;

	Auslesen: entity read
		generic map(WIDTH)
		port map(
			tcode => data_out_fifo(width-1 downto 0),
			data_valid => read_enable,
			parity_error => data_out_fifo(width),
			rd_ena => rd_ena
		);
	
end architecture behave;
